// Copyright (C) 2021 THL A29 Limited, a Tencent company.  All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software distributed under the License
// is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions and limitations under
// the License.
//
// ╔════════════════════════════════════════════════════════════════════════════════════════╗
// ║──█████████╗───███████╗───████████╗───██╗──────██╗───███████╗───████████╗───████████╗───║
// ║──██╔══════╝──██╔════██╗──██╔════██╗──██║──────██║──██╔════██╗──██╔════██╗──██╔════██╗──║
// ║──████████╗───██║────██║──████████╔╝──██║──█╗──██║──█████████║──████████╔╝──██║────██║──║
// ║──██╔═════╝───██║────██║──██╔════██╗──██║█████╗██║──██╔════██║──██╔════██╗──██║────██║──║
// ║──██║─────────╚███████╔╝──██║────██║──╚████╔████╔╝──██║────██║──██║────██║──████████╔╝──║
// ║──╚═╝──────────╚══════╝───╚═╝────╚═╝───╚═══╝╚═══╝───╚═╝────╚═╝──╚═╝────╚═╝──╚═══════╝───║
// ╚════════════════════════════════════════════════════════════════════════════════════════╝
//
// Authors: Aster JIAN (asterjian@qq.com)
//          Yzx (yzxyzxyzx777@outlook.com)
//          Ao LI (346950981@qq.com)
//          Paul LU (lujq96@gmail.com)

#pragma once

#include <memory>
#include <string>
#include <vector>

#include "fwd_keras/keras_cvt/keras_desc_creators/i_keras_layer_creator.h"

FWD_KERAS_NAMESPACE_BEGIN
/**
 * \brief FullyConnected DescCreator
 */
template <>
class TLayerDescCreator<TrtFullyConnectedDesc> : public ILayerDescCreator {
 public:
  bool Check(const Layer& layer) override {
    std::string type = layer.Type();
    return type == "Dense";
  }

  std::shared_ptr<TrtLayerDesc> Create(const Layer& layer, const H5ModelReader& reader,
                                       std::vector<std::string>& input_names) override {
    LOG(INFO) << "TrtFullyConnectDesc::Create";

    input_names = layer.Inputs();

    const std::string layer_name = layer.Name();

    auto layer_desc = std::make_shared<TrtFullyConnectedDesc>();
    layer_desc->nbOutputChannels = layer.GetAttr<int>("units");

    auto kernel_weights = reader.ReadWeight(layer_name, "kernel:0");
    layer_desc->kernelWeights = kernel_weights.ToFwdWeights();
    layer_desc->kernelWeights.Transpose(kernel_weights.GetDimension(), {1, 0});

    const bool use_bias = layer.GetAttr<bool>("use_bias");
    if (use_bias) {
      auto bias_weights = reader.ReadWeight(layer_name, "bias:0");
      layer_desc->biasWeights = bias_weights.ToFwdWeights();
    }

    return layer_desc;
  }
};

FWD_KERAS_NAMESPACE_END
